@@ -5,7 +5,7 @@
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -14,8 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  *
  * According to the definition of DES in FIPS PUB 46-2 from December 1993.
@@ -41,7 +40,7 @@
  * bits are parity bits and they will _not_ checked in this implementation, but
  * simply ignored.
  *
- * For Tripple-DES you could use either two 64bit keys or three 64bit keys.
+ * For Triple-DES you could use either two 64bit keys or three 64bit keys.
  * The parity bits will _not_ checked, too.
  *
  * After initializing a context with a key you could use this context to
@@ -141,9 +140,9 @@ working_memcmp( const char *a, const char *b, size_t n )
 /* Some defines/checks to support standalone modules */
 
 #ifndef CIPHER_ALGO_3DES
-  #define CIPHER_ALGO_3DES 2
+#define CIPHER_ALGO_3DES 2
 #elif CIPHER_ALGO_3DES != 2
-  #error CIPHER_ALGO_3DES is defined to a wrong value.
+#error CIPHER_ALGO_3DES is defined to a wrong value.
 #endif
 
 
@@ -948,7 +947,7 @@ selftest (void)
 
 
 static int
-do_tripledes_setkey ( struct _tripledes_ctx *ctx, byte *key, unsigned keylen )
+do_tripledes_setkey ( void *ctx, const byte *key, unsigned keylen )
 {
     if( selftest_failed )
 	return G10ERR_SELFTEST_FAILED;
@@ -968,14 +967,14 @@ do_tripledes_setkey ( struct _tripledes_ctx *ctx, byte *key, unsigned keylen )
 
 
 static void
-do_tripledes_encrypt( struct _tripledes_ctx *ctx, byte *outbuf, byte *inbuf )
+do_tripledes_encrypt( void *ctx, byte *outbuf, const byte *inbuf )
 {
     tripledes_ecb_encrypt ( ctx, inbuf, outbuf );
     burn_stack (32);
 }
 
 static void
-do_tripledes_decrypt( struct _tripledes_ctx *ctx, byte *outbuf, byte *inbuf )
+do_tripledes_decrypt( void *ctx, byte *outbuf, const byte *inbuf )
 {
     tripledes_ecb_decrypt ( ctx, inbuf, outbuf );
     burn_stack (32);
@@ -990,11 +989,11 @@ do_tripledes_decrypt( struct _tripledes_ctx *ctx, byte *outbuf, byte *inbuf )
  */
 const char *
 des_get_info( int algo, size_t *keylen,
-		   size_t *blocksize, size_t *contextsize,
-		   int	(**r_setkey)( void *c, byte *key, unsigned keylen ),
-		   void (**r_encrypt)( void *c, byte *outbuf, byte *inbuf ),
-		   void (**r_decrypt)( void *c, byte *outbuf, byte *inbuf )
-		 )
+	      size_t *blocksize, size_t *contextsize,
+	      int (**r_setkey)( void *c, const byte *key, unsigned keylen ),
+	      void (**r_encrypt)( void *c, byte *outbuf, const byte *inbuf ),
+	      void (**r_decrypt)( void *c, byte *outbuf, const byte *inbuf )
+	      )
 {
     static int did_selftest = 0;
 
@@ -1012,14 +1011,10 @@ des_get_info( int algo, size_t *keylen,
 	*keylen = 192;
 	*blocksize = 8;
 	*contextsize = sizeof(struct _tripledes_ctx);
-	*(int  (**)(struct _tripledes_ctx*, byte*, unsigned))r_setkey
-							= do_tripledes_setkey;
-	*(void (**)(struct _tripledes_ctx*, byte*, byte*))r_encrypt
-							= do_tripledes_encrypt;
-	*(void (**)(struct _tripledes_ctx*, byte*, byte*))r_decrypt
-							= do_tripledes_decrypt;
+	*r_setkey = do_tripledes_setkey;
+	*r_encrypt = do_tripledes_encrypt;
+	*r_decrypt = do_tripledes_decrypt;
 	return "3DES";
     }
     return NULL;
 }
-
