@@ -1,12 +1,12 @@
 /* options.h
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004,
- *               2005 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
+ *               2007 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -15,9 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 #ifndef G10_OPTIONS_H
 #define G10_OPTIONS_H
@@ -32,7 +30,7 @@
 #if defined (__riscos__) && !defined (INCLUDED_BY_MAIN_MODULE)
 #define EXTERN_UNLESS_MAIN_MODULE extern
 #else
-#define EXTERN_UNLESS_MAIN_MODULE 
+#define EXTERN_UNLESS_MAIN_MODULE
 #endif
 #endif
 
@@ -81,14 +79,18 @@ struct
   int def_cert_level;
   int min_cert_level;
   int ask_cert_level;
-  int no_version;
+  int emit_version;       /* 0 = none,
+                             1 = major only,
+                             2 = major and minor,
+                             3 = full version,
+                             4 = full version plus OS string. */
   int marginals_needed;
   int completes_needed;
   int max_cert_depth;
   const char *homedir;
 
   char *display;      /* 5 options to be passed to the gpg-agent */
-  char *ttyname;     
+  char *ttyname;
   char *ttytype;
   char *lc_ctype;
   char *lc_messages;
@@ -105,7 +107,8 @@ struct
   int force_ownertrust;
   enum
     {
-      CO_GNUPG=0, CO_RFC2440, CO_RFC1991, CO_PGP2, CO_PGP6, CO_PGP7, CO_PGP8
+      CO_GNUPG, CO_RFC4880, CO_RFC2440, CO_RFC1991, CO_PGP2,
+      CO_PGP6, CO_PGP7, CO_PGP8
     } compliance;
   enum
     {
@@ -120,8 +123,10 @@ struct
   int s2k_mode;
   int s2k_digest_algo;
   int s2k_cipher_algo;
-  int simple_sk_checksum; /* create the deprecated rfc2440 secret
-			     key protection*/
+  unsigned char s2k_count; /* This is the encoded form, not the raw
+			      count */
+  int simple_sk_checksum; /* create the deprecated rfc2440 secret key
+			     protection */
   int not_dash_escaped;
   int escape_from;
   int lock_once;
@@ -134,6 +139,12 @@ struct
     char *port;
     char *path;
     char *opaque;
+    STRLIST options;
+    struct
+    {
+      unsigned int direct_uri:1;
+    } flags;
+    struct keyserver_spec *next;
   } *keyserver;
   struct
   {
@@ -148,7 +159,8 @@ struct
   unsigned int export_options;
   unsigned int list_options;
   unsigned int verify_options;
-  char *def_preference_list;
+  const char *def_preference_list;
+  const char *def_keyserver_url;
   prefitem_t *personal_cipher_prefs;
   prefitem_t *personal_digest_prefs;
   prefitem_t *personal_compress_prefs;
@@ -157,12 +169,13 @@ struct
   char *temp_dir;
   int no_encrypt_to;
   int interactive;
-  STRLIST sig_notation_data;
-  STRLIST cert_notation_data;
+  struct notation *sig_notations;
+  struct notation *cert_notations;
   STRLIST sig_policy_url;
   STRLIST cert_policy_url;
   STRLIST sig_keyserver_url;
-  int use_embedded_filename;
+  STRLIST cert_subpackets;
+  STRLIST sig_subpackets;
   int allow_non_selfsigned_uid;
   int allow_freeform_uid;
   int no_literal;
@@ -199,17 +212,37 @@ struct
 
   /* If > 0, limit the number of card insertion prompts to this
      value. */
-  int limit_card_insert_tries; 
+  int limit_card_insert_tries;
 
 #ifdef ENABLE_CARD_SUPPORT
   const char *ctapi_driver; /* Library to access the ctAPI. */
   const char *pcsc_driver;  /* Library to access the PC/SC system. */
   int disable_ccid;    /* Disable the use of the internal CCID driver. */
+  int disable_keypad;  /* Do not allow the use of a keypad.  */
 #endif /*ENABLE_CARD_SUPPORT*/
 
-  /* True if multiple concatenated signatures may be verified. */
-  int allow_multisig_verification; 
+  struct
+  {
+    /* If set, require an 0x19 backsig to be present on signatures
+       made by signing subkeys.  If not set, a missing backsig is not
+       an error (but an invalid backsig still is). */
+    unsigned int require_cross_cert:1;
+    unsigned int use_embedded_filename:1;
+    unsigned int utf8_filename:1;
+    unsigned int dsa2:1;
+    unsigned int allow_multiple_messages:1;
+  } flags;
 
+  /* Linked list of ways to find a key if the key isn't on the local
+     keyring. */
+  struct akl
+  {
+    enum {AKL_CERT, AKL_PKA, AKL_LDAP, AKL_KEYSERVER, AKL_SPEC} type;
+    struct keyserver_spec *spec;
+    struct akl *next;
+  } *auto_key_locate;
+
+  int passwd_repeat;
 } opt;
 
 /* CTRL is used to keep some global variables we currently can't
@@ -243,14 +276,15 @@ struct {
 #define DBG_EXTPROG (opt.debug & DBG_EXTPROG_VALUE)
 #define DBG_CARD_IO (opt.debug & DBG_CARD_IO_VALUE)
 
-
 #define GNUPG   (opt.compliance==CO_GNUPG)
 #define RFC1991 (opt.compliance==CO_RFC1991 || opt.compliance==CO_PGP2)
 #define RFC2440 (opt.compliance==CO_RFC2440)
+#define RFC4880 (opt.compliance==CO_RFC4880)
 #define PGP2    (opt.compliance==CO_PGP2)
 #define PGP6    (opt.compliance==CO_PGP6)
 #define PGP7    (opt.compliance==CO_PGP7)
 #define PGP8    (opt.compliance==CO_PGP8)
+#define PGPX    (PGP2 || PGP6 || PGP7 || PGP8)
 
 /* Various option flags.  Note that there should be no common string
    names between the IMPORT_ and EXPORT_ flags as they can be mixed in
@@ -261,16 +295,16 @@ struct {
 #define IMPORT_FAST                      (1<<2)
 #define IMPORT_SK2PK                     (1<<3)
 #define IMPORT_MERGE_ONLY                (1<<4)
-#define IMPORT_CLEAN_SIGS                (1<<5)
-#define IMPORT_CLEAN_UIDS                (1<<6)
+#define IMPORT_MINIMAL                   (1<<5)
+#define IMPORT_CLEAN                     (1<<6)
+#define IMPORT_NO_SECKEY                 (1<<7)
 
 #define EXPORT_LOCAL_SIGS                (1<<0)
 #define EXPORT_ATTRIBUTES                (1<<1)
 #define EXPORT_SENSITIVE_REVKEYS         (1<<2)
-#define EXPORT_MINIMAL                   (1<<3)
-#define EXPORT_CLEAN_SIGS                (1<<4)
-#define EXPORT_CLEAN_UIDS                (1<<5)
-#define EXPORT_RESET_SUBKEY_PASSWD       (1<<6)
+#define EXPORT_RESET_SUBKEY_PASSWD       (1<<3)
+#define EXPORT_MINIMAL                   (1<<4)
+#define EXPORT_CLEAN                     (1<<5)
 
 #define LIST_SHOW_PHOTOS                 (1<<0)
 #define LIST_SHOW_POLICY_URLS            (1<<1)
@@ -293,14 +327,15 @@ struct {
 #define VERIFY_SHOW_KEYSERVER_URLS       (1<<4)
 #define VERIFY_SHOW_UID_VALIDITY         (1<<5)
 #define VERIFY_SHOW_UNUSABLE_UIDS        (1<<6)
+#define VERIFY_PKA_LOOKUPS               (1<<7)
+#define VERIFY_PKA_TRUST_INCREASE        (1<<8)
+#define VERIFY_SHOW_PRIMARY_UID_ONLY     (1<<9)
 
-#define KEYSERVER_INCLUDE_REVOKED        (1<<0)
-#define KEYSERVER_INCLUDE_SUBKEYS        (1<<1)
-#define KEYSERVER_USE_TEMP_FILES         (1<<2)
-#define KEYSERVER_KEEP_TEMP_FILES        (1<<3)
-#define KEYSERVER_ADD_FAKE_V3            (1<<4)
-#define KEYSERVER_AUTO_KEY_RETRIEVE      (1<<5)
-#define KEYSERVER_TRY_DNS_SRV            (1<<6)
-#define KEYSERVER_HONOR_KEYSERVER_URL    (1<<7)
+#define KEYSERVER_USE_TEMP_FILES         (1<<0)
+#define KEYSERVER_KEEP_TEMP_FILES        (1<<1)
+#define KEYSERVER_ADD_FAKE_V3            (1<<2)
+#define KEYSERVER_AUTO_KEY_RETRIEVE      (1<<3)
+#define KEYSERVER_HONOR_KEYSERVER_URL    (1<<4)
+#define KEYSERVER_HONOR_PKA_RECORD       (1<<5)
 
 #endif /*G10_OPTIONS_H*/
