@@ -1,11 +1,11 @@
 /* tdbio.c
- * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2012 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -14,8 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -87,7 +86,7 @@ struct cmp_xdir_struct {
 
 
 static char *db_name;
-static DOTLOCK lockhandle;
+static dotlock_t lockhandle;
 static int is_locked;
 static int  db_fd = -1;
 static int in_transaction;
@@ -187,7 +186,7 @@ put_record_into_cache( ulong recno, const char *data )
     }
     /* see whether we reached the limit */
     if( cache_entries < MAX_CACHE_ENTRIES_SOFT ) { /* no */
-	r = m_alloc( sizeof *r );
+	r = xmalloc( sizeof *r );
 	r->flags.used = 1;
 	r->recno = recno;
 	memcpy( r->data, data, TRUST_RECORD_LEN );
@@ -230,7 +229,7 @@ put_record_into_cache( ulong recno, const char *data )
 	if( cache_entries < MAX_CACHE_ENTRIES_HARD ) { /* no */
 	    if( opt.debug && !(cache_entries % 100) )
 		log_debug("increasing tdbio cache size\n");
-	    r = m_alloc( sizeof *r );
+	    r = xmalloc( sizeof *r );
 	    r->flags.used = 1;
 	    r->recno = recno;
 	    memcpy( r->data, data, TRUST_RECORD_LEN );
@@ -249,7 +248,7 @@ put_record_into_cache( ulong recno, const char *data )
 	if( !n )
 	    n = 1;
 	if( !is_locked ) {
-	    if( make_dotlock( lockhandle, -1 ) )
+	    if (dotlock_take (lockhandle, -1))
 		log_fatal("can't acquire lock - giving up\n");
 	    else
 		is_locked = 1;
@@ -268,7 +267,7 @@ put_record_into_cache( ulong recno, const char *data )
 	    }
 	}
 	if( !opt.lock_once ) {
-	    if( !release_dotlock( lockhandle ) )
+	    if (!dotlock_release (lockhandle))
 		is_locked = 0;
 	}
 	assert( unused );
@@ -310,7 +309,7 @@ tdbio_sync()
 	return 0;
 
     if( !is_locked ) {
-	if( make_dotlock( lockhandle, -1 ) )
+	if (dotlock_take (lockhandle, -1))
 	    log_fatal("can't acquire lock - giving up\n");
 	else
 	    is_locked = 1;
@@ -325,14 +324,16 @@ tdbio_sync()
     }
     cache_is_dirty = 0;
     if( did_lock && !opt.lock_once ) {
-	if( !release_dotlock( lockhandle ) )
+	if (!dotlock_release (lockhandle))
 	    is_locked = 0;
     }
 
     return 0;
 }
 
-
+#if 0
+/* The transaction code is disabled in the 1.2.x branch, as it is not
+   yet used.  It will be enabled in 1.3.x. */
 
 /****************
  * Simple transactions system:
@@ -363,7 +364,7 @@ tdbio_end_transaction()
     if( !in_transaction )
 	log_bug("tdbio: no active transaction\n");
     if( !is_locked ) {
-	if( make_dotlock( lockhandle, -1 ) )
+	if (dotlock_take (lockhandle, -1))
 	    log_fatal("can't acquire lock - giving up\n");
 	else
 	    is_locked = 1;
@@ -373,7 +374,7 @@ tdbio_end_transaction()
     rc = tdbio_sync();
     unblock_all_signals();
     if( !opt.lock_once ) {
-	if( !release_dotlock( lockhandle ) )
+	if (!dotlock_release (lockhandle))
 	    is_locked = 0;
     }
     return rc;
@@ -402,7 +403,7 @@ tdbio_cancel_transaction()
     in_transaction = 0;
     return 0;
 }
-
+#endif
 
 
 /********************************************************
@@ -413,23 +414,52 @@ static void
 cleanup(void)
 {
     if( is_locked ) {
-	if( !release_dotlock(lockhandle) )
+	if (!dotlock_release (lockhandle))
 	    is_locked = 0;
     }
 }
 
+/* Caller must sync */
+int
+tdbio_update_version_record (void)
+{
+  TRUSTREC rec;
+  int rc;
+
+  memset( &rec, 0, sizeof rec );
+
+  rc=tdbio_read_record( 0, &rec, RECTYPE_VER);
+  if(rc==0)
+    {
+      rec.r.ver.created     = make_timestamp();
+      rec.r.ver.marginals   = opt.marginals_needed;
+      rec.r.ver.completes   = opt.completes_needed;
+      rec.r.ver.cert_depth  = opt.max_cert_depth;
+      rec.r.ver.trust_model = opt.trust_model;
+      rec.r.ver.min_cert_level = opt.min_cert_level;
+      rc=tdbio_write_record(&rec);
+    }
+
+  return rc;
+}
+
 static int
 create_version_record (void)
 {
   TRUSTREC rec;
   int rc;
-  
+
   memset( &rec, 0, sizeof rec );
-  rec.r.ver.version = 3;
-  rec.r.ver.created = make_timestamp();
-  rec.r.ver.marginals  = opt.marginals_needed;
-  rec.r.ver.completes  = opt.completes_needed;
-  rec.r.ver.cert_depth = opt.max_cert_depth;
+  rec.r.ver.version     = 3;
+  rec.r.ver.created     = make_timestamp();
+  rec.r.ver.marginals   = opt.marginals_needed;
+  rec.r.ver.completes   = opt.completes_needed;
+  rec.r.ver.cert_depth  = opt.max_cert_depth;
+  if(opt.trust_model==TM_PGP || opt.trust_model==TM_CLASSIC)
+    rec.r.ver.trust_model = opt.trust_model;
+  else
+    rec.r.ver.trust_model = TM_PGP;
+  rec.r.ver.min_cert_level = opt.min_cert_level;
   rec.rectype = RECTYPE_VER;
   rec.recnum = 0;
   rc = tdbio_write_record( &rec );
@@ -441,7 +471,7 @@ create_version_record (void)
 
 
 int
-tdbio_set_dbname( const char *new_dbname, int create )
+tdbio_set_dbname( const char *new_dbname, int create, int *r_nofile)
 {
     char *fname;
     static int initialized = 0;
@@ -451,6 +481,8 @@ tdbio_set_dbname( const char *new_dbname, int create )
 	initialized = 1;
     }
 
+    *r_nofile = 0;
+
     if(new_dbname==NULL)
       fname=make_filename(opt.homedir,"trustdb" EXTSEP_S "gpg", NULL);
     else if (*new_dbname != DIRSEP_C )
@@ -461,15 +493,17 @@ tdbio_set_dbname( const char *new_dbname, int create )
 	  fname = make_filename (opt.homedir, new_dbname, NULL);
       }
     else
-      fname = m_strdup (new_dbname);
+      fname = xstrdup (new_dbname);
 
     if( access( fname, R_OK ) ) {
 	if( errno != ENOENT ) {
-	    log_error( _("%s: can't access: %s\n"), fname, strerror(errno) );
-	    m_free(fname);
+	    log_error( _("can't access `%s': %s\n"), fname, strerror(errno) );
+	    xfree(fname);
 	    return G10ERR_TRUSTDB;
 	}
-	if( create ) {
+	if (!create)
+          *r_nofile = 1;
+        else {
 	    FILE *fp;
 	    TRUSTREC rec;
 	    int rc;
@@ -480,35 +514,41 @@ tdbio_set_dbname( const char *new_dbname, int create )
 	    *p = 0;
 	    if( access( fname, F_OK ) ) {
 		try_make_homedir( fname );
-		log_fatal( _("%s: directory does not exist!\n"), fname );
+                if (access (fname, F_OK ))
+                  log_fatal (_("%s: directory does not exist!\n"), fname);
 	    }
 	    *p = DIRSEP_C;
 
-	    m_free(db_name);
+	    xfree(db_name);
 	    db_name = fname;
 #ifdef __riscos__
 	    if( !lockhandle )
-		lockhandle = create_dotlock( db_name );
+                lockhandle = dotlock_create (db_name, 0);
 	    if( !lockhandle )
-		log_fatal( _("%s: can't create lock\n"), db_name );
-            if( make_dotlock( lockhandle, -1 ) )
-                log_fatal( _("%s: can't make lock\n"), db_name );
+		log_fatal( _("can't create lock for `%s'\n"), db_name );
+            if (dotlock_take (lockhandle, -1))
+                log_fatal( _("can't lock `%s'\n"), db_name );
 #endif /* __riscos__ */
 	    oldmask=umask(077);
-	    fp =fopen( fname, "wb" );
+            if (is_secured_filename (fname)) {
+                fp = NULL;
+                errno = EPERM;
+            }
+            else
+                fp =fopen( fname, "wb" );
 	    umask(oldmask);
 	    if( !fp )
-		log_fatal( _("%s: can't create: %s\n"), fname, strerror(errno) );
+		log_fatal( _("can't create `%s': %s\n"), fname, strerror(errno) );
 	    fclose(fp);
 	    db_fd = open( db_name, O_RDWR | MY_O_BINARY );
 	    if( db_fd == -1 )
-		log_fatal( _("%s: can't open: %s\n"), db_name, strerror(errno) );
+		log_fatal( _("can't open `%s': %s\n"), db_name, strerror(errno) );
 
 #ifndef __riscos__
 	    if( !lockhandle )
-		lockhandle = create_dotlock( db_name );
+                lockhandle = dotlock_create (db_name, 0);
 	    if( !lockhandle )
-		log_fatal( _("%s: can't create lock\n"), db_name );
+		log_fatal( _("can't create lock for `%s'\n"), db_name );
 #endif /* !__riscos__ */
 
             rc = create_version_record ();
@@ -525,7 +565,7 @@ tdbio_set_dbname( const char *new_dbname, int create )
 	    return 0;
 	}
     }
-    m_free(db_name);
+    xfree(db_name);
     db_name = fname;
     return 0;
 }
@@ -549,21 +589,27 @@ open_db()
   assert( db_fd == -1 );
 
   if (!lockhandle )
-    lockhandle = create_dotlock( db_name );
+    lockhandle = dotlock_create (db_name, 0);
   if (!lockhandle )
-    log_fatal( _("%s: can't create lock\n"), db_name );
+    log_fatal( _("can't create lock for `%s'\n"), db_name );
 #ifdef __riscos__
-  if (make_dotlock( lockhandle, -1 ) )
-    log_fatal( _("%s: can't make lock\n"), db_name );
+  if (dotlock_take (lockhandle, -1))
+    log_fatal( _("can't lock `%s'\n"), db_name );
 #endif /* __riscos__ */
   db_fd = open (db_name, O_RDWR | MY_O_BINARY );
-  if (db_fd == -1 && errno == EACCES) {
+  if (db_fd == -1 && (errno == EACCES
+#ifdef EROFS
+                      || errno == EROFS
+#endif
+                      )
+      ) {
       db_fd = open (db_name, O_RDONLY | MY_O_BINARY );
       if (db_fd != -1)
           log_info (_("NOTE: trustdb not writable\n"));
   }
   if ( db_fd == -1 )
-    log_fatal( _("%s: can't open: %s\n"), db_name, strerror(errno) );
+    log_fatal( _("can't open `%s': %s\n"), db_name, strerror(errno) );
+  register_secured_file (db_name);
 
   /* check whether we need to do a version migration */
   do
@@ -573,7 +619,7 @@ open_db()
     {
       migrate_from_v2 ();
     }
-  
+
   /* read the version record */
   if (tdbio_read_record (0, &rec, RECTYPE_VER ) )
     log_fatal( _("%s: invalid trustdb\n"), db_name );
@@ -624,38 +670,40 @@ create_hashtable( TRUSTREC *vr, int type )
 int
 tdbio_db_matches_options()
 {
-    static int yes_no = -1;
+  static int yes_no = -1;
 
-    if( yes_no == -1 ) {
-	TRUSTREC vr;
-	int rc;
+  if( yes_no == -1 )
+    {
+      TRUSTREC vr;
+      int rc;
 
-	rc = tdbio_read_record( 0, &vr, RECTYPE_VER );
-	if( rc )
-	    log_fatal( _("%s: error reading version record: %s\n"),
-						    db_name, g10_errstr(rc) );
+      rc = tdbio_read_record( 0, &vr, RECTYPE_VER );
+      if( rc )
+	log_fatal( _("%s: error reading version record: %s\n"),
+		   db_name, g10_errstr(rc) );
 
-	if( !vr.r.ver.marginals && !vr.r.ver.completes
-				&& !vr.r.ver.cert_depth )
-	{   /* special hack for trustdbs created by old versions of GnuPG */
-	    vr.r.ver.marginals =  opt.marginals_needed;
-	    vr.r.ver.completes =  opt.completes_needed;
-	    vr.r.ver.cert_depth = opt.max_cert_depth;
-	    rc = tdbio_write_record( &vr );
-	    if( !rc && !in_transaction )
-		rc = tdbio_sync();
-	    if( rc )
-		log_error( _("%s: error writing version record: %s\n"),
-						db_name, g10_errstr(rc) );
-	}
-
-	yes_no = vr.r.ver.marginals == opt.marginals_needed
-		 && vr.r.ver.completes == opt.completes_needed
-		 && vr.r.ver.cert_depth == opt.max_cert_depth;
+      yes_no = vr.r.ver.marginals == opt.marginals_needed
+	&& vr.r.ver.completes == opt.completes_needed
+	&& vr.r.ver.cert_depth == opt.max_cert_depth
+	&& vr.r.ver.trust_model == opt.trust_model
+	&& vr.r.ver.min_cert_level == opt.min_cert_level;
     }
-    return yes_no;
+
+  return yes_no;
 }
 
+byte
+tdbio_read_model(void)
+{
+  TRUSTREC vr;
+  int rc;
+
+  rc = tdbio_read_record( 0, &vr, RECTYPE_VER );
+  if( rc )
+    log_fatal( _("%s: error reading version record: %s\n"),
+	       db_name, g10_errstr(rc) );
+  return vr.r.ver.trust_model;
+}
 
 /****************
  * Return the nextstamp value.
@@ -744,8 +792,7 @@ upd_hashtable( ulong table, byte *key, int keylen, ulong newrecnum )
     hashrec += msb / ITEMS_PER_HTBL_RECORD;
     rc = tdbio_read_record( hashrec, &rec, RECTYPE_HTBL );
     if( rc ) {
-	log_error( db_name, "upd_hashtable: read failed: %s\n",
-							g10_errstr(rc) );
+	log_error("upd_hashtable: read failed: %s\n",	g10_errstr(rc) );
 	return rc;
     }
 
@@ -754,7 +801,7 @@ upd_hashtable( ulong table, byte *key, int keylen, ulong newrecnum )
 	rec.r.htbl.item[msb % ITEMS_PER_HTBL_RECORD] = newrecnum;
 	rc = tdbio_write_record( &rec );
 	if( rc ) {
-	    log_error( db_name, "upd_hashtable: write htbl failed: %s\n",
+	    log_error("upd_hashtable: write htbl failed: %s\n",
 							    g10_errstr(rc) );
 	    return rc;
 	}
@@ -893,7 +940,7 @@ drop_from_hashtable( ulong table, byte *key, int keylen, ulong recnum )
     hashrec += msb / ITEMS_PER_HTBL_RECORD;
     rc = tdbio_read_record( hashrec, &rec, RECTYPE_HTBL );
     if( rc ) {
-	log_error( db_name, "drop_from_hashtable: read failed: %s\n",
+	log_error("drop_from_hashtable: read failed: %s\n",
 							g10_errstr(rc) );
 	return rc;
     }
@@ -906,7 +953,7 @@ drop_from_hashtable( ulong table, byte *key, int keylen, ulong recnum )
 	rec.r.htbl.item[msb % ITEMS_PER_HTBL_RECORD] = 0;
 	rc = tdbio_write_record( &rec );
 	if( rc )
-	    log_error( db_name, "drop_from_hashtable: write htbl failed: %s\n",
+	    log_error("drop_from_hashtable: write htbl failed: %s\n",
 							    g10_errstr(rc) );
 	return rc;
     }
@@ -935,7 +982,7 @@ drop_from_hashtable( ulong table, byte *key, int keylen, ulong recnum )
 		    rec.r.hlst.rnum[i] = 0; /* drop */
 		    rc = tdbio_write_record( &rec );
 		    if( rc )
-			log_error( db_name, "drop_from_hashtable: write htbl failed: %s\n",
+			log_error("drop_from_hashtable: write htbl failed: %s\n",
 									g10_errstr(rc) );
 		    return rc;
 		}
@@ -968,8 +1015,8 @@ drop_from_hashtable( ulong table, byte *key, int keylen, ulong recnum )
  */
 static int
 lookup_hashtable( ulong table, const byte *key, size_t keylen,
-		  int (*cmpfnc)(void*, const TRUSTREC *), void *cmpdata,
-						TRUSTREC *rec )
+		  int (*cmpfnc)(const void*, const TRUSTREC *),
+                  const void *cmpdata, TRUSTREC *rec )
 {
     int rc;
     ulong hashrec, item;
@@ -982,7 +1029,7 @@ lookup_hashtable( ulong table, const byte *key, size_t keylen,
     hashrec += msb / ITEMS_PER_HTBL_RECORD;
     rc = tdbio_read_record( hashrec, rec, RECTYPE_HTBL );
     if( rc ) {
-	log_error( db_name, "lookup_hashtable failed: %s\n", g10_errstr(rc) );
+	log_error("lookup_hashtable failed: %s\n", g10_errstr(rc) );
 	return rc;
     }
 
@@ -992,14 +1039,14 @@ lookup_hashtable( ulong table, const byte *key, size_t keylen,
 
     rc = tdbio_read_record( item, rec, 0 );
     if( rc ) {
-	log_error( db_name, "hashtable read failed: %s\n", g10_errstr(rc) );
+	log_error( "hashtable read failed: %s\n", g10_errstr(rc) );
 	return rc;
     }
     if( rec->rectype == RECTYPE_HTBL ) {
 	hashrec = item;
 	level++;
 	if( level >= keylen ) {
-	    log_error( db_name, "hashtable has invalid indirections\n");
+	    log_error("hashtable has invalid indirections\n");
 	    return G10ERR_TRUSTDB;
 	}
 	goto next_level;
@@ -1069,12 +1116,14 @@ tdbio_dump_record( TRUSTREC *rec, FILE *fp  )
       case 0: fprintf(fp, "blank\n");
 	break;
       case RECTYPE_VER: fprintf(fp,
-	    "version, td=%lu, f=%lu, m/c/d=%d/%d/%d nc=%lu (%s)\n",
+	    "version, td=%lu, f=%lu, m/c/d=%d/%d/%d tm=%d mcl=%d nc=%lu (%s)\n",
                                    rec->r.ver.trusthashtbl,
 				   rec->r.ver.firstfree,
 				   rec->r.ver.marginals,
 				   rec->r.ver.completes,
 				   rec->r.ver.cert_depth,
+				   rec->r.ver.trust_model,
+				   rec->r.ver.min_cert_level,
                                    rec->r.ver.nextcheck,
 				   strtimestamp(rec->r.ver.nextcheck)
                                  );
@@ -1167,10 +1216,9 @@ tdbio_read_record( ulong recnum, TRUSTREC *rec, int expected )
 	rec->r.ver.marginals = *p++;
 	rec->r.ver.completes = *p++;
 	rec->r.ver.cert_depth = *p++;
-	/* Warning - one byte of this is used in the devel version.
-	   If you change this, make sure to consult the same code in
-	   devel. */
-	p += 4; /* lock flags */
+	rec->r.ver.trust_model = *p++;
+	rec->r.ver.min_cert_level = *p++;
+	p += 2;
 	rec->r.ver.created  = buftoulong(p); p += 4;
 	rec->r.ver.nextcheck = buftoulong(p); p += 4;
 	p += 4;
@@ -1207,8 +1255,9 @@ tdbio_read_record( ulong recnum, TRUSTREC *rec, int expected )
 	memcpy( rec->r.trust.fingerprint, p, 20); p+=20;
         rec->r.trust.ownertrust = *p++;
         rec->r.trust.depth = *p++;
-        p += 2;
-	rec->r.trust.validlist  = buftoulong(p); p += 4;
+        rec->r.trust.min_ownertrust = *p++;
+        p++;
+	rec->r.trust.validlist = buftoulong(p); p += 4;
 	break;
       case RECTYPE_VALID:
 	memcpy( rec->r.valid.namehash, p, 20); p+=20;
@@ -1255,10 +1304,9 @@ tdbio_write_record( TRUSTREC *rec )
 	*p++ = rec->r.ver.marginals;
 	*p++ = rec->r.ver.completes;
 	*p++ = rec->r.ver.cert_depth;
-	/* Warning - one byte of this is used in the devel version.
-	   If you change this, make sure to consult the same code in
-	   devel. */
-	p += 4; /* skip lock flags */
+	*p++ = rec->r.ver.trust_model;
+	*p++ = rec->r.ver.min_cert_level;
+	p += 2;
 	ulongtobuf(p, rec->r.ver.created); p += 4;
 	ulongtobuf(p, rec->r.ver.nextcheck); p += 4;
 	p += 4;
@@ -1290,7 +1338,8 @@ tdbio_write_record( TRUSTREC *rec )
 	memcpy( p, rec->r.trust.fingerprint, 20); p += 20;
 	*p++ = rec->r.trust.ownertrust;
 	*p++ = rec->r.trust.depth;
-        p += 2;
+	*p++ = rec->r.trust.min_ownertrust;
+        p++;
 	ulongtobuf( p, rec->r.trust.validlist); p += 4;
 	break;
 
@@ -1426,10 +1475,10 @@ tdbio_new_recnum()
 
 
 static int
-cmp_trec_fpr ( void *fpr, const TRUSTREC *rec )
+cmp_trec_fpr (const void *fpr, const TRUSTREC *rec )
 {
-    return rec->rectype == RECTYPE_TRUST
-	   && !memcmp( rec->r.trust.fingerprint, fpr, 20);
+    return (rec->rectype == RECTYPE_TRUST
+            && !memcmp( rec->r.trust.fingerprint, fpr, 20));
 }
 
 
@@ -1438,9 +1487,9 @@ tdbio_search_trust_byfpr( const byte *fingerprint, TRUSTREC *rec )
 {
     int rc;
 
-    /* locate the trust record using the hash table */
-    rc = lookup_hashtable( get_trusthashrec(), fingerprint, 20,
-			   cmp_trec_fpr, (void*)fingerprint, rec );
+    /* Locate the trust record using the hash table.  */
+    rc = lookup_hashtable (get_trusthashrec(), fingerprint, 20,
+			   cmp_trec_fpr, fingerprint, rec);
     return rc;
 }
 
@@ -1461,9 +1510,9 @@ tdbio_search_trust_bypk (PKT_public_key *pk, TRUSTREC *rec)
 void
 tdbio_invalid(void)
 {
-    log_error(_(
-	"the trustdb is corrupted; please run \"gpg --fix-trustdb\".\n") );
-    g10_exit(2);
+  log_error (_("Error: The trustdb is corrupted.\n"));
+  how_to_fix_the_trustdb ();
+  g10_exit (2);
 }
 
 /*
@@ -1489,18 +1538,18 @@ migrate_from_v2 ()
   int rc, count;
 
   ottable_size = 5;
-  ottable = m_alloc (ottable_size * sizeof *ottable);
+  ottable = xmalloc (ottable_size * sizeof *ottable);
   ottable_used = 0;
 
   /* We have some restrictions here.  We can't use the version record
    * and we can't use any of the old hashtables because we dropped the
    * code.  So we first collect all ownertrusts and then use a second
-   * pass fo find the associated keys.  We have to do this all without using 
+   * pass fo find the associated keys.  We have to do this all without using
    * the regular record read functions.
    */
 
   /* get all the ownertrusts */
-  if (lseek (db_fd, 0, SEEK_SET ) == -1 ) 
+  if (lseek (db_fd, 0, SEEK_SET ) == -1 )
       log_fatal ("migrate_from_v2: lseek failed: %s\n", strerror (errno));
   for (recno=0;;recno++)
     {
@@ -1514,12 +1563,12 @@ migrate_from_v2 ()
 
       if (*oldbuf != 2)
         continue;
-      
+
       /* v2 dir record */
       if (ottable_used == ottable_size)
         {
           ottable_size += 1000;
-          ottable = m_realloc (ottable, ottable_size * sizeof *ottable);
+          ottable = xrealloc (ottable, ottable_size * sizeof *ottable);
         }
       ottable[ottable_used].keyrecno = buftoulong (oldbuf+6);
       ottable[ottable_used].ot = oldbuf[18];
@@ -1531,7 +1580,7 @@ migrate_from_v2 ()
   log_info ("found %d ownertrust records\n", ottable_used);
 
   /* Read again and find the fingerprints */
-  if (lseek (db_fd, 0, SEEK_SET ) == -1 ) 
+  if (lseek (db_fd, 0, SEEK_SET ) == -1 )
       log_fatal ("migrate_from_v2: lseek failed: %s\n", strerror (errno));
   for (recno=0;;recno++)
     {
@@ -1543,7 +1592,7 @@ migrate_from_v2 ()
       if (n != 40)
         log_fatal ("migrate_from_v2: read error or short read\n");
 
-      if (*oldbuf != 3) 
+      if (*oldbuf != 3)
         continue;
 
       /* v2 key record */
@@ -1564,7 +1613,7 @@ migrate_from_v2 ()
   if (create_version_record ())
     log_fatal ("failed to recreate version record of `%s'\n", db_name);
 
-  /* access the hash table, so it is store just after the version record, 
+  /* access the hash table, so it is store just after the version record,
    * this is not needed put a dump is more pretty */
   get_trusthashrec ();
 
@@ -1574,7 +1623,7 @@ migrate_from_v2 ()
     {
       if (!ottable[i].okay)
         continue;
-      
+
       memset (&rec, 0, sizeof rec);
       rec.recnum = tdbio_new_recnum ();
       rec.rectype = RECTYPE_TRUST;
@@ -1590,8 +1639,5 @@ migrate_from_v2 ()
   if (rc)
     log_fatal ("failed to sync `%s'\n", db_name);
   log_info ("migrated %d version 2 ownertrusts\n", count);
-  m_free (ottable);
+  xfree (ottable);
 }
-
-
-
