@@ -20,9 +20,7 @@
 #ifndef G10_UTIL_H
 #define G10_UTIL_H
 
-#if defined (_WIN32) || defined (__CYGWIN32__)
 #include <stdarg.h>
-#endif
 
 #include "types.h"
 #include "errors.h"
@@ -51,6 +49,7 @@ typedef struct {
 	 const char *last;
 	 void *aliases;
 	 const void *cur_alias;
+         void *iio_list;
      } internal;	    /* DO NOT CHANGE */
 } ARGPARSE_ARGS;
 
@@ -126,15 +125,7 @@ const char *strusage( int level );
 
 
 /*-- dotlock.c --*/
-struct dotlock_handle;
-typedef struct dotlock_handle *DOTLOCK;
-
-void disable_dotlock(void);
-DOTLOCK create_dotlock( const char *file_to_lock );
-void destroy_dotlock ( DOTLOCK h );
-int make_dotlock( DOTLOCK h, long timeout );
-int release_dotlock( DOTLOCK h );
-void remove_lockfiles (void);
+#include "../include/dotlock.h"
 
 /*-- fileutil.c --*/
 char * make_basename(const char *filepath, const char *inputpath);
@@ -219,16 +210,17 @@ int strncasecmp (const char *, const char *b, size_t n);
 /* The definition of the structure is private, we only need it here,
    so it can be allocated on the stack. */
 struct private_membuf_s {
-  size_t len;      
-  size_t size;     
-  char *buf;       
-  int out_of_core; 
+  size_t len;
+  size_t size;
+  char *buf;
+  int out_of_core;
 };
 
 typedef struct private_membuf_s membuf_t;
 
 void init_membuf (membuf_t *mb, int initiallen);
 void put_membuf  (membuf_t *mb, const void *buf, size_t len);
+void put_membuf_str (membuf_t *mb, const char *buf);
 void *get_membuf (membuf_t *mb, size_t *len);
 
 
@@ -245,6 +237,7 @@ int write_w32_registry_string(const char *root, const char *dir,
 /*-- strgutil.c --*/
 char *xasprintf (const char *fmt, ...);
 char *xtryasprintf (const char *fmt, ...);
+char *xtryvasprintf (const char *fmt, va_list arg_ptr);
 
 
 /*-- pka.c --*/
