@@ -1,13 +1,14 @@
 /* curl-shim.c - Implement a small subset of the curl API in terms of
  * the iobuf HTTP API
  *
- * Copyright (C) 2005, 2006 Free Software Foundation, Inc.
+ * Copyright (C) 2005, 2006, 2007, 2008, 2009, 2012,
+ *               2013 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -16,9 +17,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -131,20 +130,26 @@ curl_easy_setopt(CURL *curl,CURLoption option,...)
       curl->proxy=va_arg(ap,char *);
       break;
     case CURLOPT_POST:
-      curl->flags.post=va_arg(ap,unsigned int);
+      curl->flags.post=va_arg(ap,long)?1:0;
       break;
     case CURLOPT_POSTFIELDS:
       curl->postfields=va_arg(ap,char *);
       break;
+    case CURLOPT_SRVTAG_GPG_HACK:
+      curl->srvtag=va_arg(ap,char *);
+      break;
     case CURLOPT_FAILONERROR:
-      curl->flags.failonerror=va_arg(ap,unsigned int);
+      curl->flags.failonerror=va_arg(ap,long)?1:0;
       break;
     case CURLOPT_VERBOSE:
-      curl->flags.verbose=va_arg(ap,unsigned int);
+      curl->flags.verbose=va_arg(ap,long)?1:0;
       break;
     case CURLOPT_STDERR:
       curl->errors=va_arg(ap,FILE *);
       break;
+    case CURLOPT_HTTPHEADER:
+      curl->headers=va_arg(ap,struct curl_slist *);
+      break;
     default:
       /* We ignore the huge majority of curl options */
       break;
@@ -160,6 +165,9 @@ curl_easy_perform(CURL *curl)
   CURLcode err=CURLE_OK;
   const char *errstr=NULL;
   char *proxy=NULL;
+  struct http_srv srv;
+
+  memset(&srv,0,sizeof(srv));
 
   /* Emulate the libcurl proxy behavior.  If the calling program set a
      proxy, use it.  If it didn't set a proxy or set it to NULL, check
@@ -172,17 +180,36 @@ curl_easy_perform(CURL *curl)
   else
     proxy=getenv(HTTP_PROXY_ENV);
 
+  if(curl->srvtag)
+    srv.srvtag=curl->srvtag;
+
   if(curl->flags.verbose)
-    fprintf(curl->errors,"* HTTP proxy is \"%s\"\n",proxy?proxy:"null");
+    {
+      fprintf(curl->errors,"* HTTP proxy is \"%s\"\n",proxy?proxy:"null");
+      fprintf(curl->errors,"* HTTP URL is \"%s\"\n",curl->url);
+      if(srv.srvtag)
+	fprintf(curl->errors,
+		"* SRV tag is \"%s\": host and port may be overridden\n",
+		srv.srvtag);
+      fprintf(curl->errors,"* HTTP auth is \"%s\"\n",
+	      curl->auth?curl->auth:"null");
+      fprintf(curl->errors,"* HTTP method is %s\n",
+	      curl->flags.post?"POST":"GET");
+    }
 
   if(curl->flags.post)
     {
-      rc=http_open(&curl->hd,HTTP_REQ_POST,curl->url,curl->auth,0,proxy);
+      rc=http_open(&curl->hd,HTTP_REQ_POST,curl->url,curl->auth,0,proxy,
+		   &srv,curl->headers?curl->headers->list:NULL);
       if(rc==0)
 	{
 	  char content_len[50];
 	  unsigned int post_len=strlen(curl->postfields);
 
+	  if(curl->flags.verbose && srv.used_server && srv.used_port)
+	    fprintf (curl->errors, "* HTTP host:port post-SRV is \"%s:%hu\"\n",
+		     srv.used_server, srv.used_port);
+
 	  iobuf_writestr(curl->hd.fp_write,
 			 "Content-Type: application/x-www-form-urlencoded\r\n");
 	  sprintf(content_len,"Content-Length: %u\r\n",post_len);
@@ -198,9 +225,14 @@ curl_easy_perform(CURL *curl)
     }
   else
     {
-      rc=http_open(&curl->hd,HTTP_REQ_GET,curl->url,curl->auth,0,proxy);
+      rc=http_open(&curl->hd,HTTP_REQ_GET,curl->url,curl->auth,0,proxy,
+		   &srv,curl->headers?curl->headers->list:NULL);
       if(rc==0)
 	{
+	  if(curl->flags.verbose && srv.used_server && srv.used_port)
+	    fprintf (curl->errors, "* HTTP host:port post-SRV is \"%s:%hu\"\n",
+		     srv.used_server, srv.used_port);
+
 	  rc=http_wait_response(&curl->hd,&curl->status);
 	  if(rc==0)
 	    {
@@ -235,6 +267,8 @@ curl_easy_perform(CURL *curl)
 	}
     }
 
+  free (srv.used_server);
+
   switch(rc)
     {
     case 0:
@@ -258,6 +292,27 @@ curl_easy_perform(CURL *curl)
   return handle_error(curl,err,errstr);
 }
 
+CURLcode
+curl_easy_getinfo(CURL *curl, CURLINFO info, ... )
+{
+  va_list ap;
+  long *var;
+
+  va_start(ap,info);
+
+  switch(info)
+    {
+    case CURLINFO_RESPONSE_CODE:
+      var=va_arg(ap,long *);
+      *var=curl->status;
+      break;
+    default:
+      break;
+    }
+
+  return handle_error(curl,CURLE_OK,NULL);
+}
+
 /* This is not the same exact set that is allowed according to
    RFC-2396, but it is what the real curl uses. */
 #define VALID_URI_CHARS "abcdefghijklmnopqrstuvwxyz" \
@@ -265,7 +320,7 @@ curl_easy_perform(CURL *curl)
                         "0123456789"
 
 char *
-curl_escape(char *str,int length)
+curl_easy_escape(CURL *curl,char *str,int length)
 {
   int len,max,idx,enc_idx=0;
   char *enc;
@@ -275,7 +330,7 @@ curl_escape(char *str,int length)
   else
     len=strlen(str);
 
-  enc=malloc(len+1);
+  enc = xtrymalloc(len+1);
   if(!enc)
     return enc;
 
@@ -315,8 +370,38 @@ curl_escape(char *str,int length)
   return enc;
 }
 
-void
-curl_free(char *ptr)
+curl_version_info_data *
+curl_version_info(int type)
 {
-  free(ptr);
+  static curl_version_info_data data;
+  static const char *protocols[]={"http",NULL};
+
+  data.protocols=protocols;
+
+  return &data;
+}
+
+struct curl_slist *
+curl_slist_append(struct curl_slist *list,const char *string)
+{
+  if(!list)
+    {
+      list=calloc(1,sizeof(*list));
+      if(!list)
+	return NULL;
+    }
+
+  add_to_strlist(&list->list,string);
+
+  return list;
+}
+
+void
+curl_slist_free_all(struct curl_slist *list)
+{
+  if(list)
+    {
+      free_strlist(list->list);
+      free(list);
+    }
 }
