@@ -6,7 +6,7 @@
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -15,9 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -33,6 +31,17 @@
 #include "util.h"
 #include "i18n.h"
 
+#ifndef HAVE_TIMEGM
+time_t timegm (struct tm *tm);
+#endif
+
+#ifdef HAVE_UNSIGNED_TIME_T
+# define INVALID_TIME_CHECK(a) ((a) == (time_t)(-1))
+#else
+  /* Error or 32 bit time_t and value after 2038-01-19.  */
+# define INVALID_TIME_CHECK(a) ((a) < 0)
+#endif
+
 /****************
  * I know that the OpenPGP protocol has a Y2106 problem ;-)
  */
@@ -119,8 +128,8 @@ strtimestamp( u32 stamp )
     static char buffer[11+5];
     struct tm *tp;
     time_t atime = stamp;
-    
-    if (atime < 0) {
+
+    if (INVALID_TIME_CHECK (atime)) {
         strcpy (buffer, "????" "-??" "-??");
     }
     else {
@@ -141,8 +150,8 @@ isotimestamp (u32 stamp)
     static char buffer[25+5];
     struct tm *tp;
     time_t atime = stamp;
-    
-    if (atime < 0) {
+
+    if (INVALID_TIME_CHECK (atime)) {
         strcpy (buffer, "????" "-??" "-??" " " "??" ":" "??" ":" "??");
     }
     else {
@@ -154,6 +163,57 @@ isotimestamp (u32 stamp)
     return buffer;
 }
 
+
+/* Scan am ISO timestamp and return an Epoch based timestamp.  The only
+   supported format is "yyyymmddThhmmss" delimited by white space, nul, a
+   colon or a comma.  Returns 0 for an invalid string. */
+u32
+isotime2seconds (const char *string)
+{
+  const char *s;
+  int year, month, day, hour, minu, sec;
+  struct tm tmbuf;
+  int i;
+  time_t result;
+
+  if (!*string)
+    return 0;
+  for (s=string, i=0; i < 8; i++, s++)
+    if (!digitp (s))
+      return 0;
+  if (*s != 'T')
+      return 0;
+  for (s++, i=9; i < 15; i++, s++)
+    if (!digitp (s))
+      return 0;
+  if ( !(!*s || (isascii (*s) && isspace(*s)) || *s == ':' || *s == ','))
+    return 0;  /* Wrong delimiter.  */
+
+  year  = atoi_4 (string);
+  month = atoi_2 (string + 4);
+  day   = atoi_2 (string + 6);
+  hour  = atoi_2 (string + 9);
+  minu  = atoi_2 (string + 11);
+  sec   = atoi_2 (string + 13);
+
+  /* Basic checks.  */
+  if (year < 1970 || month < 1 || month > 12 || day < 1 || day > 31
+      || hour > 23 || minu > 59 || sec > 61 )
+    return 0;
+
+  memset (&tmbuf, 0, sizeof tmbuf);
+  tmbuf.tm_sec  = sec;
+  tmbuf.tm_min  = minu;
+  tmbuf.tm_hour = hour;
+  tmbuf.tm_mday = day;
+  tmbuf.tm_mon  = month-1;
+  tmbuf.tm_year = year - 1900;
+  tmbuf.tm_isdst = -1;
+  result = timegm (&tmbuf);
+  return (result == (time_t)(-1))? 0 : (u32)result;
+}
+
+
 /****************
  * Note: this function returns local time
  */
@@ -167,25 +227,33 @@ asctimestamp( u32 stamp )
     struct tm *tp;
     time_t atime = stamp;
 
-    if (atime < 0) {
+    if (INVALID_TIME_CHECK (atime))
+      {
         strcpy (buffer, "????" "-??" "-??");
         return buffer;
-    }
+      }
 
     tp = localtime( &atime );
 #ifdef HAVE_STRFTIME
 #if defined(HAVE_NL_LANGINFO)
+# if GNUPG_GCC_VERSION >= 40600
+#  pragma GCC diagnostic push
+#  pragma GCC diagnostic ignored "-Wformat-nonliteral"
+# endif
       mem2str( fmt, nl_langinfo(D_T_FMT), DIM(fmt)-3 );
       if( strstr( fmt, "%Z" ) == NULL )
 	strcat( fmt, " %Z");
       strftime( buffer, DIM(buffer)-1, fmt, tp );
+# if GNUPG_GCC_VERSION >= 40600
+#  pragma GCC diagnostic pop
+# endif
 #else
       /* fixme: we should check whether the locale appends a " %Z"
        * These locales from glibc don't put the " %Z":
        * fi_FI hr_HR ja_JP lt_LT lv_LV POSIX ru_RU ru_SU sv_FI sv_SE zh_CN
        */
-      strftime( buffer, DIM(buffer)-1, 
-#ifdef HAVE_W32_SYSTEM                
+      strftime( buffer, DIM(buffer)-1,
+#ifdef HAVE_W32_SYSTEM
                 "%c"
 #else
                 "%c %Z"
@@ -207,7 +275,8 @@ void
 print_string2( FILE *fp, const byte *p, size_t n, int delim, int delim2 )
 {
     for( ; n; n--, p++ )
-	if( *p < 0x20 || (*p >= 0x7f && *p < 0xa0)
+	if (*p < 0x20
+            || *p == 0x7f
 	    || *p == delim || *p == delim2
 	    || ((delim || delim2) && *p=='\\'))
 	  {
@@ -313,7 +382,7 @@ make_printable_string( const byte *p, size_t n, int delim )
 		*d++ = '0';
 	    else {
 		sprintf(d, "x%02x", *p );
-		d += 2;
+		d += 3;
 	    }
 	}
 	else
@@ -326,10 +395,10 @@ make_printable_string( const byte *p, size_t n, int delim )
 int
 answer_is_yes_no_default( const char *s, int def_answer )
 {
-    /* NOTE TO TRANSLATOR: See doc/TRANSLATE about this string. */
+    /* TRANSLATORS: See doc/TRANSLATE about this string. */
     const char *long_yes = _("yes");
     const char *short_yes = _("yY");
-    /* NOTE TO TRANSLATOR: See doc/TRANSLATE about this string. */
+    /* TRANSLATORS: See doc/TRANSLATE about this string. */
     const char *long_no = _("no");
     const char *short_no = _("nN");
 
@@ -363,11 +432,11 @@ answer_is_yes( const char *s )
 int
 answer_is_yes_no_quit( const char *s )
 {
-    /* NOTE TO TRANSLATOR: See doc/TRANSLATE about this string. */
+    /* TRANSLATORS: See doc/TRANSLATE about this string. */
     const char *long_yes = _("yes");
-    /* NOTE TO TRANSLATOR: See doc/TRANSLATE about this string. */
+    /* TRANSLATORS: See doc/TRANSLATE about this string. */
     const char *long_no = _("no");
-    /* NOTE TO TRANSLATOR: See doc/TRANSLATE about this string. */
+    /* TRANSLATORS: See doc/TRANSLATE about this string. */
     const char *long_quit = _("quit");
     const char *short_yes = _("yY");
     const char *short_no = _("nN");
@@ -398,18 +467,18 @@ answer_is_yes_no_quit( const char *s )
 }
 
 /*
-   Return 1 for okay, 0 for for cancel or DEF_ANSWER for default. 
+   Return 1 for okay, 0 for for cancel or DEF_ANSWER for default.
  */
 int
 answer_is_okay_cancel (const char *s, int def_answer)
 {
-  /* NOTE TO TRANSLATOR: See doc/TRANSLATE about this string. */
+  /* TRANSLATORS: See doc/TRANSLATE about this string. */
   const char *long_okay = _("okay|okay");
-  /* NOTE TO TRANSLATOR: See doc/TRANSLATE about this string. */
+  /* TRANSLATORS: See doc/TRANSLATE about this string. */
   const char *long_cancel = _("cancel|cancel");
   const char *short_okay = _("oO");
   const char *short_cancel = _("cC");
-  
+
   /* Note: We have to use the locale dependent strcasecmp */
   if ( match_multistr(long_okay,s) )
     return 1;
@@ -453,28 +522,3 @@ match_multistr(const char *multistr,const char *match)
 
   return 0;
 }
-
-int
-hextobyte( const char *s )
-{
-    int c;
-
-    if( *s >= '0' && *s <= '9' )
-	c = 16 * (*s - '0');
-    else if( *s >= 'A' && *s <= 'F' )
-	c = 16 * (10 + *s - 'A');
-    else if( *s >= 'a' && *s <= 'f' )
-	c = 16 * (10 + *s - 'a');
-    else
-	return -1;
-    s++;
-    if( *s >= '0' && *s <= '9' )
-	c += *s - '0';
-    else if( *s >= 'A' && *s <= 'F' )
-	c += 10 + *s - 'A';
-    else if( *s >= 'a' && *s <= 'f' )
-	c += 10 + *s - 'a';
-    else
-	return -1;
-    return c;
-}
